string GlobalVersion = "1.0";

/*
                 Creates arrows from ADX and SmoothedROC indicators
   
*/


//---- indicator settings
#property  indicator_separate_window
#property  indicator_minimum 0.0
#property  indicator_maximum 100.0
#property  indicator_buffers 8
#property  indicator_color1  Navy
#property  indicator_color2  Red
#property  indicator_color3  Purple
#property  indicator_width1  1
//---- indicator parameters
extern int Period1=21;
extern int Period2=21;
extern int Period3=30;     // period3 always =30, unless calculating for different time frames
extern int PeriodCross=8;
extern int PeriodADX=14;
//---- indicator buffers
double     SmoothedROCBuffer[];
double     SignalBuffer[];
double     ADXBuffer[];

bool       FirstTime = true;
int        IndicatorSubWindow = 0;
int        ArrowCount = 0, DuplicateArrowCount = 0, 
           PreviousCounted = 0, LastAlertTime = 0;
string     SubWindowName;
string     SmoothedROCIndicatorName = "SmoothedROC",
           ADXIndicatorName = "ADX";


int deinit()
{
   if(IndicatorSubWindow > 0)
      ObjectsDeleteAll(IndicatorSubWindow);
}


//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {


  
//---- drawing settings
//   SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexStyle(0,DRAW_LINE);
   SetIndexStyle(1,DRAW_LINE,STYLE_DOT);
   SetIndexStyle(2,DRAW_NONE);
//   SetIndexDrawBegin(1,PeriodCross);
   IndicatorDigits(Digits+4);
//---- indicator buffers mapping
   SetIndexBuffer(0,SmoothedROCBuffer);
   SetIndexBuffer(1,SignalBuffer);
   SetIndexBuffer(2,ADXBuffer);
//---- name for DataWindow and indicator subwindow label

   SubWindowName = StringConcatenate("ROCTrigger Version ", GlobalVersion, "   (", Period1, " ", Period2, " ", PeriodCross, ")      ");
   IndicatorShortName(SubWindowName);
   SetIndexLabel(0,"SmoothedROC");
   SetIndexLabel(1,"Signal");
   SetIndexLabel(2,"ADX");

   ArrowCount = 0;
   DuplicateArrowCount = 0;
   LastAlertTime = 0;
//---- initialization done
   return(0);
  }
  
  

string FormatTimeStamp(int TimeStamp)
{
   string Output;
   if(TimeStamp == 0) return(" ");
   
     
   Output =         TimeYear(TimeStamp) +
            "/"  +  TimeMonth(TimeStamp) +
            "/"  +  TimeDay(TimeStamp) +
            " "  +  TimeHour(TimeStamp) +
            ":"  +  TimeMinute(TimeStamp);

   

   return(Output);
   
}


string FormatDate(int TimeStamp)
{
   string Output;
   if(TimeStamp == 0) return(" ");
     
   Output =         TimeDay(TimeStamp) +
            "/"  +  TimeMonth(TimeStamp) +
            "/"  +  TimeYear(TimeStamp);

   return(Output);
   
}
  
  
  
//+------------------------------------------------------------------+
//                       |
//+------------------------------------------------------------------+
int start()
  {
   int i, limit, LastError = 0, ArrowCode = 0, NearestCandle=0, DeleteCount;
   int counted_bars=IndicatorCounted();
   double OldEma = 0;
   string  Name15, Name25, Name85, TriggerName;
   

   
   if(FirstTime == true)
   {
      FirstTime = false;  
      Print("Starting ROCTrigger, Counted=", counted_bars, " of total candles=", Bars);
              
      IndicatorSubWindow = WindowFind(SubWindowName);
      Name15 = "Level15-" + IndicatorSubWindow;   
//    Name25 = "Level25-" + IndicatorSubWindow;   
      Name85 = "Level85-" + IndicatorSubWindow;
      
      ObjectCreate(Name85,OBJ_TREND, IndicatorSubWindow, Time[Bars], 85.0, Time[0], 85.0);
      ObjectSet   (Name85,OBJPROP_COLOR,LightSlateGray);
      ObjectCreate(Name15,OBJ_TREND, IndicatorSubWindow, Time[Bars], 15.0, Time[0], 15.0);
      ObjectSet   (Name15,OBJPROP_COLOR,LightSlateGray);

      Print("Create lines in window ", IndicatorSubWindow, " last error=", GetLastError() );       
   }
   else     
      if(IndicatorCounted() == 0)
         {   
            Print("M", Period(), " Total=", Bars, " counted = ", IndicatorCounted(), " Delete and ReCreate All Arrows");
            if(IndicatorSubWindow > 0)
             {    
                  DeleteCount = ObjectsDeleteAll(IndicatorSubWindow, OBJ_ARROW);
                  Print(DeleteCount, " arrows deleted");
//                Alert(Symbol(), " Need to rescan chart for SmoothedROC + ADX");
              }
         }

   
   
   
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   limit=Bars-counted_bars;

/*
   if(IndicatorCounted() == 0 || IndicatorCounted() != PreviousCounted )
   {
      PreviousCounted = IndicatorCounted();
      Print("M", Period(), " Total=", Bars, " counted = ", IndicatorCounted(), " Arrows=",  ArrowCount,
            " Repeats= ", DuplicateArrowCount, 
            " Start=", FormatTimeStamp(Time[Bars-1]));
   }
*/   
//   Print ("Total Bars = ", Bars, " counted = ", IndicatorCounted());

   if(limit == 0)  return(0);

 

   for(i=0; i<limit; i++)
      {
   
         SmoothedROCBuffer[i]= iCustom(NULL, 0, SmoothedROCIndicatorName, Period1, Period2, PeriodCross, Period3, 0, i);
         LastError = GetLastError();
         if(LastError > 0)
            {
               Print("Error calling ", SmoothedROCIndicatorName, " = ", LastError);
            }  
            
//         if (i < 5)
//               Print("Returned SmoothedROC value ", i, " = ", SmoothedROCBuffer[i]);   
            
            
         SignalBuffer[i]= iCustom(NULL, 0, SmoothedROCIndicatorName, Period1, Period2, PeriodCross, 1, i);
         LastError = GetLastError();
         if(LastError > 0)
            {
               Print("Error calling ", SmoothedROCIndicatorName, " = ", LastError);
            }  
            
//         if (i < 5)
//               Print("Returned signal value ", i, " = ", SignalBuffer[i]);   

            
         ADXBuffer[i]= iCustom(NULL, 0, ADXIndicatorName, PeriodADX, 0, i);
         LastError = GetLastError();
         if(LastError > 0)
            {
               Print("Error calling ", ADXIndicatorName, " = ", LastError);
            }  
            
//         if (i < 5)
//               Print("Returned ADX value ", i, " = ", ADXBuffer[i]);   
            
      }


   bool FirstTriggerDone = false;
     for(i=0; i<limit; i++)
         {     
// look for SmoothedROC crossing down thru signal within overbought with ADX > 25 and ADX decreasing
         if(     SmoothedROCBuffer[i+1]  > 85  
             &&  SmoothedROCBuffer[i]    <= SignalBuffer[i]     
             &&  SmoothedROCBuffer[i+1]  >  SignalBuffer[i+1] 
             &&  (SignalBuffer[i+1] > 85 || SignalBuffer[i] > 85)
             &&  ADXBuffer  [i]    >  25
             &&  ADXBuffer  [i+1]  >  ADXBuffer   [i]
             )
         {  
            if( (SignalBuffer[i] - SmoothedROCBuffer[i]) < (SmoothedROCBuffer[i+1] - SignalBuffer[i+1]))
               NearestCandle = i;     // find nearest candle 
            else
               NearestCandle = i+1;  
                
            TriggerName = "Trigger" + FormatTimeStamp(Time[i]);
            
            LastError = 0;
            if(ObjectCreate(TriggerName,OBJ_ARROW,IndicatorSubWindow,Time[NearestCandle] ,SmoothedROCBuffer[NearestCandle] + 0.0) == false)
                {  
                  LastError = GetLastError();
                  if(LastError == 4200)
                     {
                        Print("M", Period(), " Duplicate arrow from repeat scan.");
                        DuplicateArrowCount ++;
                    
                     }
                  else
                        Print("Object for ROCTrigger Cross Down Failed", LastError);
                }

            if (LastError == 0)
            {
                ArrowCode = 222;            //  222 = down arrow in circle  
                ObjectSet(TriggerName,OBJPROP_ARROWCODE,ArrowCode) ;
                ObjectSet(TriggerName,OBJPROP_COLOR,Red);
                ObjectSet(TriggerName,OBJPROP_WIDTH,2); 
                ArrowCount ++;
            } 
            
            if(FirstTriggerDone == false)
                {
                     FirstTriggerDone = true;
                     if(Time[i] != LastAlertTime)  // avoid repeat alerts 
                     {
                        LastAlertTime = Time[i];
                        Alert(Symbol(), " ", FormatDate(Time[i]), " SmoothedROC XOver Down and ADX");
                     }
                }
           }               
           
//  look for SmoothedROC crossing up while in oversold with ADX > 25 and ADX decreasing         
         if(     SmoothedROCBuffer[i+1]  <  15         
             &&  SmoothedROCBuffer[i]    >  SignalBuffer[i]     
             &&  SmoothedROCBuffer[i+1]  <= SignalBuffer[i+1]
             &&  (SignalBuffer[i+1] < 15 || SignalBuffer[i] < 15)             
             &&  ADXBuffer  [i]    >  25
             &&  ADXBuffer  [i+1]  >  ADXBuffer   [i]
           )
           
         {             
              if( (SmoothedROCBuffer[i] - SignalBuffer[i]) < (SignalBuffer[i+1] - SmoothedROCBuffer[i+1]) )
                  NearestCandle = i;     // find nearest candle 
               else
                  NearestCandle = i+1;   

               TriggerName = "Trigger" + FormatTimeStamp(Time[i]);
               
               LastError = 0;
               if(ObjectCreate(TriggerName,OBJ_ARROW,IndicatorSubWindow,Time[NearestCandle], SmoothedROCBuffer[NearestCandle] + 12.0) == false)
               
                { 
                  LastError = GetLastError();
                  if(LastError == 4200)
                     {
                        Print("M", Period(), " Duplicate arrow from repeat scan.");
                        DuplicateArrowCount ++;
               
                     }
                  else                
                       Print("Object for SmoothedROC Cross Down Failed", GetLastError());
                }
         
               if(LastError == 0)
                  {
                     ArrowCode = 221;            //  221 = up arrow in circle  
                     ObjectSet(TriggerName,OBJPROP_ARROWCODE,ArrowCode) ;
                     ObjectSet(TriggerName,OBJPROP_COLOR,C'0,185,92');               // custom green 
                     ObjectSet(TriggerName,OBJPROP_WIDTH,2); 
                     ArrowCount ++;               
                  }
                  
            if(FirstTriggerDone == false)
                {
                     FirstTriggerDone = true;
                     if(Time[i] != LastAlertTime)  // avoid repeat alerts 
                     {
                        LastAlertTime = Time[i];
                        Alert(Symbol(), " ", FormatDate(Time[i]), " SmoothedROC XOver Up   and ADX");
                      }
                }
                   
         }     
         

      }

//---- done
   return(0);
  }
//+------------------------------------------------------------------+